@@ -21,7 +21,16 @@ class Poster():
             uid=self.uid,
         ))
 
+    async def post_failure(self, message):
+        await self.send(dict(
+            t='f',
+            e=message,
+            uid=self.uid,
+        ))
+
     async def post_row(self, phase, index, row, errors=None):
+        if '__errors' in row:
+            errors = row.pop('__errors')
         await self.send(dict(
             t='r',
             p=row,
@@ -38,3 +47,10 @@ class Poster():
             i=index,
             uid=self.uid,
         ))
+
+    async def post_done(self, phase):
+        await self.send(dict(
+            t='d',
+            j=phase,
+            uid=self.uid,
+        ))
